﻿using System.Collections.Generic;
using Eco.Gameplay.Components;
using Eco.Gameplay.Components.Auth;
using Eco.Gameplay.DynamicValues;
using Eco.Gameplay.Items;
using Eco.Gameplay.Objects;
using Eco.Gameplay.Property;
using Eco.Gameplay.Skills;
using Eco.Gameplay.Systems.TextLinks;
using Eco.Gameplay.Systems.Tooltip;
using Eco.Mods.TechTree;
using Eco.Shared.Localization;
using Eco.Shared.Math;
using Eco.Shared.Serialization;

namespace FZMModsLibrary.Artisty
{
    [Serialized]
    [RequireComponent(typeof(AttachmentComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(MinimapComponent))]
    [RequireComponent(typeof(LinkComponent))]
    [RequireComponent(typeof(CraftingComponent))]
    [RequireComponent(typeof(SolidGroundComponent))]
    [RequireComponent(typeof(RoomRequirementsComponent))]
    [RequireRoomContainment]
    [RequireRoomVolume(25)]
    public partial class ArtStationObject : WorldObject
    {
        public override string FriendlyName { get { return "Art Station"; } }

        static ArtStationObject()
        {
            WorldObject.AddOccupancy<ArtStationObject>(new List<BlockOccupancy>(){
            new BlockOccupancy(new Vector3i(0, 0, 0)),
            new BlockOccupancy(new Vector3i(0, 1, 0)),
            new BlockOccupancy(new Vector3i(-1, 0, 0)),
            new BlockOccupancy(new Vector3i(-1, 1, 0)),
            });
        }

        protected override void Initialize() 
        {
            this.GetComponent<MinimapComponent>().Initialize("Crafting");
        }

        public override void Destroy() { base.Destroy(); }
    }

    [Serialized]
    public partial class ArtStationItem : WorldObjectItem<ArtStationObject>
    {
        public override string FriendlyName { get { return "Art Station"; } }
        public override string Description { get { return "For creative snowflakes."; } }

        static ArtStationItem() { }
    }

    [RequiresSkill(typeof(CanvasPaintingSkill), 1)]
    public partial class ArtStationRecipe : Recipe
    {
        public ArtStationRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<ArtStationItem>(),
            };

            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<LogItem>(typeof(CanvasPaintingEfficiencySkill), 20, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<ClothItem>(typeof(CanvasPaintingEfficiencySkill), 20, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<PaintBrushItem>(typeof(CanvasPaintingEfficiencySkill), 20, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
            };
            SkillModifiedValue value = new SkillModifiedValue(5, CanvasPaintingSpeedSkill.MultiplicativeStrategy, typeof(CanvasPaintingSpeedSkill), Localizer.Do("craft time"));
            SkillModifiedValueManager.AddBenefitForObject(typeof(ArtStationRecipe), Item.Get<ArtStationItem>().UILink(), value);
            SkillModifiedValueManager.AddSkillBenefit(Item.Get<ArtStationItem>().UILink(), value);
            this.CraftMinutes = value;
            this.Initialize("Art Station", typeof(ArtStationRecipe));
            CraftingComponent.AddRecipe(typeof(CarpentryTableObject), this);
        }
    }
}
