﻿using Eco.Gameplay.Components;
using Eco.Gameplay.Components.Auth;
using Eco.Gameplay.Housing;
using Eco.Gameplay.Items;
using Eco.Gameplay.Objects;
using Eco.Gameplay.Skills;
using Eco.Gameplay.Systems.TextLinks;
using Eco.Gameplay.Systems.Tooltip;
using Eco.Mods.TechTree;
using Eco.Shared.Serialization;

namespace FZMModsLibrary.Artisty
{
    [Serialized]
    [RequireComponent(typeof(AttachmentComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(MinimapComponent))]
    [RequireComponent(typeof(HousingComponent))]
    public partial class TTTCanvasObject :
        WorldObject
    {
        public override string FriendlyName { get { return "TinyTina Treats Canvas"; } }

        protected override void Initialize()
        {
            this.GetComponent<MinimapComponent>().Initialize("Housing");
            this.GetComponent<HousingComponent>().Set(TTTCanvasItem.HousingVal);
        }

        public override void Destroy() { base.Destroy(); }
    }

    [RequiresSkill(typeof(CanvasPaintingSkill), 2)]
    public partial class TTTCanvasRecipe : Recipe
    {
        public TTTCanvasRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<TTTCanvasItem>(),
            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<EmptyCanvasItem>(1),
                new CraftingElement<PaintBrushItem>(typeof(CanvasPaintingEfficiencySkill), 5, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<PalettePaintsItem>(typeof(CanvasPaintingEfficiencySkill), 5, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<BasicEngineeringSkillScroll>(1),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(TTTCanvasRecipe), Item.Get<TTTCanvasItem>().UILink(), 30.0f, typeof(CanvasPaintingSpeedSkill));
            this.Initialize("TinyTina Treats Canvas", typeof(TTTCanvasRecipe));

            CraftingComponent.AddRecipe(typeof(ArtStationObject), this);
        }
    }

    [Serialized]
    [Weight(500)]
    public partial class TTTCanvasItem : WorldObjectItem<TTTCanvasObject>
    {
        public override string FriendlyName { get { return "TinyTina Treats Canvas"; } }
        public override string Description { get { return "By some unknown artist."; } }

        static TTTCanvasItem() { }

        [TooltipChildren] public HousingValue HousingTooltip { get { return HousingVal; } }
        [TooltipChildren]
        public static HousingValue HousingVal
        {
            get
            {
                return new HousingValue()
                {
                    Category = "General",
                    Val = 2,
                    TypeForRoomLimit = "Wall Painting",
                    DiminishingReturnPercent = 0.6f
                };
            }
        }
    }

    [Serialized]
    [RequireComponent(typeof(AttachmentComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(MinimapComponent))]
    [RequireComponent(typeof(HousingComponent))]
    public partial class ExcavatorRaceCanvasObject :
        WorldObject
    {
        public override string FriendlyName { get { return "Excavator Race Canvas"; } }

        protected override void Initialize()
        {
            this.GetComponent<MinimapComponent>().Initialize("Housing");
            this.GetComponent<HousingComponent>().Set(ExcavatorRaceCanvasItem.HousingVal);
        }

        public override void Destroy() { base.Destroy(); }
    }

    [RequiresSkill(typeof(CanvasPaintingSkill), 3)]
    public partial class ExcavatorRaceCanvasRecipe : Recipe
    {
        public ExcavatorRaceCanvasRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<ExcavatorRaceCanvasItem>(),
            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<EmptyCanvasItem>(1),
                new CraftingElement<PaintBrushItem>(typeof(CanvasPaintingEfficiencySkill), 5, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<PalettePaintsItem>(typeof(CanvasPaintingEfficiencySkill), 5, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<MechanicsSkillScroll>(1),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(ExcavatorRaceCanvasRecipe), Item.Get<ExcavatorRaceCanvasItem>().UILink(), 30.0f, typeof(CanvasPaintingSpeedSkill));
            this.Initialize("Excavator Race Canvas", typeof(ExcavatorRaceCanvasRecipe));

            CraftingComponent.AddRecipe(typeof(ArtStationObject), this);
        }
    }

    [Serialized]
    [Weight(500)]
    public partial class ExcavatorRaceCanvasItem : WorldObjectItem<ExcavatorRaceCanvasObject>
    {
        public override string FriendlyName { get { return "Excavator Race Canvas"; } }
        public override string Description { get { return "By some unknown artist."; } }

        static ExcavatorRaceCanvasItem() { }

        [TooltipChildren] public HousingValue HousingTooltip { get { return HousingVal; } }
        [TooltipChildren]
        public static HousingValue HousingVal
        {
            get
            {
                return new HousingValue()
                {
                    Category = "General",
                    Val = 2,
                    TypeForRoomLimit = "Wall Painting",
                    DiminishingReturnPercent = 0.6f
                };
            }
        }
    }

    [Serialized]
    [RequireComponent(typeof(AttachmentComponent))]
    [RequireComponent(typeof(PropertyAuthComponent))]
    [RequireComponent(typeof(MinimapComponent))]
    [RequireComponent(typeof(HousingComponent))]
    public partial class StreetLightsCanvasObject :
        WorldObject
    {
        public override string FriendlyName { get { return "Street Lights Canvas"; } }

        protected override void Initialize()
        {
            this.GetComponent<MinimapComponent>().Initialize("Housing");
            this.GetComponent<HousingComponent>().Set(StreetLightsCanvasItem.HousingVal);
        }

        public override void Destroy() { base.Destroy(); }
    }

    [RequiresSkill(typeof(CanvasPaintingSkill), 4)]
    public partial class StreetLightsCanvasRecipe : Recipe
    {
        public StreetLightsCanvasRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<StreetLightsCanvasItem>(),
            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<EmptyCanvasItem>(1),
                new CraftingElement<PaintBrushItem>(typeof(CanvasPaintingEfficiencySkill), 5, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<PalettePaintsItem>(typeof(CanvasPaintingEfficiencySkill), 5, CanvasPaintingEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<ElectronicsSkillScroll>(1),
            };
            this.CraftMinutes = CreateCraftTimeValue(typeof(StreetLightsCanvasRecipe), Item.Get<StreetLightsCanvasItem>().UILink(), 30.0f, typeof(CanvasPaintingSpeedSkill));
            this.Initialize("Street Lights Canvas", typeof(StreetLightsCanvasRecipe));

            CraftingComponent.AddRecipe(typeof(ArtStationObject), this);
        }
    }

    [Serialized]
    [Weight(500)]
    public partial class StreetLightsCanvasItem : WorldObjectItem<StreetLightsCanvasObject>
    {
        public override string FriendlyName { get { return "Street Lights Canvas"; } }
        public override string Description { get { return "By some unknown artist."; } }

        static StreetLightsCanvasItem() { }

        [TooltipChildren] public HousingValue HousingTooltip { get { return HousingVal; } }
        [TooltipChildren]
        public static HousingValue HousingVal
        {
            get
            {
                return new HousingValue()
                {
                    Category = "General",
                    Val = 2,
                    TypeForRoomLimit = "Wall Painting",
                    DiminishingReturnPercent = 0.6f
                };
            }
        }
    }
}
